# encoding: utf-8

import os
import subprocess
import xml.etree.cElementTree as ET
from tempfile import mkstemp
from xml.parsers.expat import ExpatError

from pkg_resources import resource_filename

CLASSPATH = ':'.join([
    resource_filename('codaxmli', 'java/router.jar'),
    resource_filename('codaxmli', 'java/xercesImpl.jar'),
])


class Client(object):

    def __init__(self, host, port, lsv_name, user, password, company,
                 trace=False):
        self.host = host
        self.port = port
        self.lsv_name = lsv_name
        self.user = user
        self.password = password
        self.company = company
        self.trace = trace

    def send(self, request):
        if not isinstance(request, basestring):
            request = ET.tostring(request)
        if self.trace:
            print request
        request_fd, request_path = mkstemp(prefix='xmli-req-')
        os.write(request_fd, request)
        os.close(request_fd)

        response_fd, response_path = mkstemp(prefix='xmli-resp-')
        error_fd, error_path = mkstemp(prefix='xmli-err-')
        subprocess.check_call([
            'java', '-classpath', CLASSPATH, 'com.coda.xml.router.Router',
            '-nsv:' + self.host, '-port:' + str(self.port),
            '-name:' + self.lsv_name, '-user:' + self.user,
            '-password:' + self.password, '-company:' + self.company,
            request_path
        ], stdout=response_fd, stderr=error_fd)
        os.close(response_fd)
        os.close(error_fd)

        try:
            response = ET.parse(response_path)
        except (ExpatError, SyntaxError):
            with open(error_path) as error_file:
                error_str = error_file.read()
            if 'COMMON_FW_INVALID_USER' in error_str:
                raise LoginFailed()
            else:
                raise XMLiError(error_str)

        os.remove(request_path)
        os.remove(response_path)
        os.remove(error_path)

        # Enlever les namespaces de la réponse. C'est plus facile à traiter
        # et Coda accepte de toute façon les requêtes sans les namespaces.
        for elem in response.getiterator():
            if elem.tag.startswith('{'):
                ns_end = elem.tag.find('}')
                if ns_end != -1:
                    elem.tag = elem.tag[ns_end + 1:]

        errors = response.findall('//Reason')
        if errors:
            msgs = []
            for error in errors:
                lines = []
                for text in error.findall('Text'):
                    lines.append(text.text)
                for path in error.findall('Path'):
                    lines.append('Path: ' + path.text)
                for hint in error.findall('Hint'):
                    lines.append(hint.text)
                msgs.append('\n'.join(lines))
            raise XMLiError('\n\n'.join(msgs).encode('utf-8'))

        return response.getroot()


class XMLiError(Exception):
    pass


class LoginFailed(XMLiError):

    def __init__(self):
        super(XMLiError, self).__init__('Login failed')
